'''
Created on 2020-09-15

@author: wf
'''
from corpus.eventcorpus import EventDataSource, EventDataSourceConfig
from corpus.event import EventStorage,EventSeriesManager, EventSeries, Event, EventManager
from lodstorage.storageconfig import StorageConfig
import re
import datetime

class GND(EventDataSource):
    '''
    manages event data from Gemeinsame Normdatei
    https://d-nb.info/standards/elementset/gnd
    '''
    debug=False
    host="confident.dbis.rwth-aachen.de"
    endpoint=f"https://{host}/jena/gnd/sparql"
    limit=1000000
    sourceConfig = EventDataSourceConfig(lookupId="gnd", name="GND", url='https://d-nb.info/standards/elementset/gnd', title='Gemeinsame Normdatei', tableSuffix="gnd")
    
    def __init__(self,debug=False):
        '''
        constructor
        '''
        super().__init__(GndEventManager(), GndEventSeriesManager(), GND.sourceConfig)
        self.debug=debug
        
    @staticmethod
    def strToDate(dateStr,debug:bool=False):
        '''
        convert the given string to a date
        
        Args:
            dateStr(str): the date string to convert
            debug(bool): if True show debug information
            
        Return:
            Date: - the converted data but None if there is a ValueError on conversion
        '''
        result=None
        try:
            result=datetime.datetime.strptime(
                        dateStr, "%d.%m.%Y").date()
        except ValueError as ve:
            # TODO year might be ok but not garbage such as
            if debug:
                print(f"{dateStr}:{str(ve)}")
            pass
        return result
        
    @staticmethod
    def getDateRange(date):
        '''
        given a GND date string create a date range
        
        Args:
            date(str): the date string to analyze
            
        Returns:
            dict: containing year, startDate, endDate
        examples:
        2018-2019
        08.01.2019-11.01.2019
        2019
        '''
        result={}
        if date is not None:
            startDate=None
            endDate=None
            yearPattern="[12][0-9]{3}"
            datePattern="[0-9]{2}[.][0-9]{2}[.]"+yearPattern
            yearOnly=re.search(r"^("+yearPattern+")[-]?("+yearPattern+")?$",date)
            if yearOnly:
                result['year']=int(yearOnly.group(1))
            else:
                fromOnly=re.search(r"^("+datePattern+")[-]?$",date)
                if fromOnly:
                    startDate=GND.strToDate(fromOnly.group(1))
                else:
                    fromTo=re.search(r"^("+datePattern+")[-]("+datePattern+")$",date)
                    if fromTo:
                        startDate=GND.strToDate(fromTo.group(1))
                        endDate=GND.strToDate(fromTo.group(2))
            if startDate:
                result['startDate']=startDate
            if endDate:
                result['endDate']=endDate
        if 'startDate' in result:
                result['year']=result['startDate'].year
        return result

        
class GndEvent(Event):
    '''
    a scientific event derived from Gemeinsame Normdatei
    '''
    @classmethod
    def getSamples(cls):
        samples = [      
        ]
        return samples
    
    def setOrganization(self,orgStr):
        self.organization=orgStr
    
    def setLocation(self,location):
        self.location=location
        
    def setOrdinal(self,ordinalStr,counter):
        ordinalStr=ordinalStr.strip()
        m=re.match(r"^([0-9]+)\.?$",ordinalStr)
        if m:
            ordinal=int(m.group(1))
            self.ordinal=ordinal
            pass
        else:
            if GND.debug:
                print(f"ordinal?:{ordinalStr}")
            counter["invalidOrdinal"]+=1
    
    def setYear(self,yearStr,counter):
        '''
        set the year of the given event
        '''
        yearStr=yearStr.strip()
        if re.match(r"^[0-9]+$",yearStr):
            self.year=int(yearStr)
            return True
        else: 
            m=re.match(r"^([0-9]+)\s*\-\s*([0-9]+)$",yearStr)
            if m:
                self.year=int(m.group(1))
                counter["yearRange"]+=1
                return True
            else:
                if GND.debug:
                    print(f"year?:{yearStr}")
                counter["invalidYear"]+=1
                return False
        
    def titleExtract(self,counter):
        '''
        extract meta data information from the title of this gnd event
        '''
        regex=r"(.*?)\((.*?)\)"
        m=re.match(regex,self.title)
        if m:
            self.title=m.group(1)
            ordYearLocation=m.group(2)
            parts=ordYearLocation.split(":")
            plen=len(parts)
            counter[plen]+=1
            if plen==1:
                self.setYear(ordYearLocation,counter)
            elif plen==2:
                self.setYear(parts[0],counter)
                self.setLocation(parts[1])
            elif plen==3:
                self.setOrdinal(parts[0],counter)
                self.setYear(parts[1],counter)
                self.setLocation(parts[2])
            elif plen==4:
                self.setOrganization(parts[0])
                self.setOrdinal(parts[1],counter)
                self.setYear(parts[2],counter)
                self.setLocation(parts[3])
                pass
        else:
            #print(event.title)
            counter["invalid"]+=1
      
class GndEventSeries(EventSeries):
    '''
    a scientific event series derived from Gemeinsame Normdatei
    '''
    
        
class GndEventManager(EventManager):
    '''
    manage WikiCFP derived scientific events
    '''

    def __init__(self, config:StorageConfig=None):
        '''
        Constructor
        '''
        self.source="gnd"
        self.endpoint=GND.endpoint
        super().__init__(name="GndEvents", sourceConfig=GND.sourceConfig,clazz=GndEvent, config=config)
    
    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            self.getListOfDicts=self.getLoDfromEndpoint
    
    def getSparqlQuery(self):
        '''
        get  the SPARQL query for this event manager
        '''
        sparql="""# performance optimized query of GND event details
# with aggregated properties as single, count and | separated list column
# WF 2021-12-05
PREFIX gndi:  <https://d-nb.info/gnd>
PREFIX gnd:  <https://d-nb.info/standards/elementset/gnd#>
PREFIX gndo: <https://d-nb.info/standards/vocab/gnd/>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX dc: <http://purl.org/dc/terms/>
PREFIX wdrs: <http://www.w3.org/2007/05/powder-s#>

SELECT  
   ?event 
   ?eventId  
   (MIN(?eventTitle) as ?title)

   (COUNT (DISTINCT ?eventDate) as ?dateCount)
   (MIN(?eventDate) as ?date)

   (MIN(?eventAcronym) as ?acronym)
   (COUNT (DISTINCT ?eventAcronym) as ?acronymCount)
   (GROUP_CONCAT(DISTINCT ?eventAcronym; SEPARATOR="| ") AS ?acronyms)

   (MIN(?eventVariant) as ?variant)
   (COUNT (DISTINCT ?eventVariant) as ?variantCount)
   (GROUP_CONCAT(DISTINCT ?eventVariant; SEPARATOR="| ") AS ?variants) 

   (MIN(?eventPlace) as ?place)
   (COUNT (DISTINCT ?eventPlace) as ?placeCount)
   (GROUP_CONCAT(DISTINCT ?eventPlace; SEPARATOR="| ") AS ?places) 

   (MIN(?eventHomepage) as ?homepage)
WHERE {
  ?event a gnd:ConferenceOrEvent.
  ?event gnd:gndIdentifier ?eventId.
  ?event gnd:preferredNameForTheConferenceOrEvent ?eventTitle.
  OPTIONAL { ?event gnd:abbreviatedNameForTheConferenceOrEvent ?eventAcronym. }
  OPTIONAL { ?event gnd:homepage ?eventHomepage. }
  OPTIONAL { ?event gnd:variantNameForTheConferenceOrEvent ?eventVariant. }
  OPTIONAL { ?event gnd:dateOfConferenceOrEvent ?eventDate. }
  OPTIONAL { ?event gnd:placeOfConferenceOrEvent ?eventPlace }
  # only available 3520 times 2021-12
  # ?event gnd:topic ?topic.
  # only available 12106 times 2021-12
  # ?event gnd:precedingConferenceOrEvent ?prec
  # only available 11929 times 2021-12
  #?event gnd:succeedingConferenceOrEvent ?succ
}
GROUP BY ?event ?eventId
LIMIT %d""" % (GND.limit)
        return sparql

class GndEventSeriesManager(EventSeriesManager):
    '''
    manage GND derived scientific conference series
    '''

    def __init__(self, config:StorageConfig=None):
        '''
        Constructor
        '''
        self.source="gnd"
        self.endpoint=GND.endpoint
        super().__init__(name="GndEventSeries", sourceConfig=GND.sourceConfig, clazz=GndEventSeries, config=config)

    def configure(self):
        '''
        configure me
        '''
        if not hasattr(self, "getListOfDicts"):
            #self.getListOfDicts=self.getLoDfromEndpoint  
            self.getListOfDicts=self.getDummyLoD
            
    def getDummyLoD(self):
        lod=[
            {
                "seriesId":"0",
                "title":""
            }
        ]
        return lod
        
    def getSparqlQuery(self):
        '''
        get  the SPARQL query for this event series manager
        '''
        sparql="""SELECT ?s ?p ?o 
WHERE {
  VALUES (?s) { ("subject") }
  VALUES (?p) { ("predicate") }
  VALUES (?o) { ("object") }
}
        """
        return sparql
        


'''
Created on 2021-07-21

@author: wf
'''
import datetime
from typing import Callable
from lodstorage.entity import EntityManager
from lodstorage.jsonable import JSONAble
from wikibot.wikiuser import WikiUser
from wikibot.wikiclient import WikiClient
from wikibot.wikipush import WikiPush
from wikifile.wikiFileManager import WikiFileManager
from wikifile.wikiFile import WikiFile
from pathlib import Path

import time


class SMWEntity(object):
    '''
    an Entity stored in Semantic MediaWiki in WikiSon notation
    '''

    def __init__(self, entity:JSONAble, wikiFile:WikiFile=None):
        '''
        Constructor
        '''
        self.entity=entity
        self._wikiFile = wikiFile
        self._wikiFileManager = wikiFile.wikiFileManager if wikiFile else None

    @property
    def wikiFile(self):
        if self._wikiFile:
            return self._wikiFile
        else:
            pageTitle = getattr(self.entity, "pageTitle")
            wikiMarkup=getattr(self.entity, "wikiMarkup") if hasattr(self.entity, "wikiMarkup") else ""
            self._wikiFile=WikiFile(pageTitle, wikiFileManager=self.wikiFileManager, wikiText=wikiMarkup)
            return self._wikiFile

    @wikiFile.setter
    def wikiFile(self, wikiFile:WikiFile):
        '''Sets the wikiFile and overwrites the wikiMarkup of the entity'''
        self._wikiFile=wikiFile
        if hasattr(self.entity, "wikiMarkup"):
            setattr(self.entity,"wikiMarkup", wikiFile.wikiText)

    @property
    def wikiFileManager(self):
        return self._wikiFileManager

    @wikiFileManager.setter
    def wikiFileManager(self, wikiFileManager:WikiFileManager):
        self._wikiFileManager=wikiFileManager

    @staticmethod
    def updateDictKeys( record: dict, lookup: dict, reverseLookup:bool=False) -> dict:
        '''
        Updates the keys of the given record based on the given lookup dict.
        The key of the given lookup dict identifies the old key and the value the new key

        Args:
            record(dict): the record to be updated
            lookup(dict): the mapping of keys/names for the new key names
            reverseLookup(bool): If true the lookup is reversed
        Return:
            dict: given record with updated keys
        '''
        reverseDict = lambda d: {value:key for key, value in d.items()}
        result = None
        if record:
            result = {}
            if reverseLookup:
                lookup=reverseDict(lookup)
            for oldKey, newKey in lookup.items():
                if oldKey in record:
                    result[newKey] = record[oldKey]
        return result

    def updateWikiText(self, overwrite:bool=False):
        """Updates the WikiSon notation in the WikiText/WikiFile with the records of this entity"""
        wikiSonRecord = self.entity.__dict__.copy()
        lookup = self.entity.getTemplateParamLookup()
        if lookup:
            wikiSonRecord = self.updateDictKeys(wikiSonRecord, lookup, reverseLookup=True)
        # handle datetime to date (if only date do not add 00:00:00 as default time)
        for key, value in wikiSonRecord.items():
            if isinstance(value, datetime.datetime):
                if value.hour==0 and value.minute==0:
                    wikiSonRecord[key]=value.date()
        self.wikiFile.updateTemplate(self.entity.templateName, wikiSonRecord, prettify=True, overwrite=overwrite)

    def saveToWikiText(self, overwrite:bool=False):
        '''
        Saves the entity to a wikiText file. The wikiSon in the files is updated with the values of the entity
  
        Args:
            overwrite(bool): If True existing files might be overwritten
        '''
        self.updateWikiText(overwrite=overwrite)
        self.wikiFile.save_to_file(overwrite=overwrite)

    def pushToWiki(self, msg:str=None, overwrite:bool=False, wikiFileManager:WikiFileManager=None):
        """
        Pushes the wikiMarkup of this entity to the target wiki of the assigned wikiFileManager

        Args:
            msg(str): message to show on as comment on the pushed changes
            overwrite(bool): If True existing files might be overwritten
            wikiFileManager(WikiFileManager): Overwrites the wikiFileManager of this object if not None
        """
        if wikiFileManager:
            self.wikiFileManager=wikiFileManager
            if self._wikiFile:
                self.wikiFile.wikiFileManager=self.wikiFileManager
        self.updateWikiText(overwrite=overwrite)
        self.wikiFile.pushToWiki(msg=msg)



class SMWEntityList(object):
    '''
    Semantic MediaWiki backed entity list
    '''

    def __init__(self, entityManager:EntityManager, wikiFileManager:WikiFileManager=None):
        '''
        constructor
        '''
        self.entityManager=entityManager
        self.profile = False
        self.debug = False
        self.wikiClient = None
        self.wikiPush = None
        self.wikiFileManager = wikiFileManager
        self.askExtra = ""

    @classmethod
    def getDefaultCachePath(cls):
        '''
        get the defaultPath to my cache
        '''
        home = str(Path.home())
        cachedir = f"{home}/.smw"
        return cachedir

    def updateEntity(self, entity, identifier='pageTitle'):
        '''
        Add/Update the given entity in the entityList
        '''
        if hasattr(entity, identifier):
            attributes = [*entity.__dict__]
            for origEntity in self.entityManager.getList():
                if origEntity.pageTitle == entity.pageTitle:
                    origAttributes = [*origEntity.__dict__]
                    difference = set(attributes) - set(origAttributes)
                    for attr in difference:
                        setattr(origEntity, attr, getattr(entity, attr))
                    return
            self.entityManager.getList().append(entity)
        else:
            raise Exception('identifier not found in entity given')

    def getAskQuery(self, askExtra="", propertyLookupList=None):
        '''
        get the query that will ask for all my events

        Args:
           askExtra(str): any additional SMW ask query constraints
           propertyLookupList:  a list of dicts for propertyLookup

        Return:
            str: the SMW ask query
        '''
        entityName = self.entityManager.entityName
        selector = "IsA::%s" % entityName
        ask = """{{#ask:[[%s]]%s
|mainlabel=pageTitle
|?Creation date=creationDate
|?Modification date=modificationDate
|?Last editor is=lastEditor
""" % (selector, askExtra)
        if propertyLookupList is None:
            if self.entityManager is not None and hasattr(self.entityManager.clazz, 'propertyLookupList'):
                propertyLookupList = getattr(self.entityManager.clazz, 'propertyLookupList')
        for propertyLookup in propertyLookupList:
            propName = propertyLookup['prop']
            name = propertyLookup['name']
            ask += "|?%s=%s\n" % (propName, name)
        ask += "}}"
        return ask

    def fromWiki(self, wikiuser:WikiUser, askExtra="", profile=False):
        '''
        read me from a wiki using the given WikiUser configuration
        '''
        records=self.getLoDfromWiki(wikiuser,askExtra,profile)
        self.entityManager.fromLoD(records)
        return records

    def getLoDfromWiki(self, wikiuser:WikiUser, askExtra="", profile=False):
        if self.wikiClient is None:
            self.wikiClient = WikiClient.ofWikiUser(wikiuser)
            self.wikiPush = WikiPush(fromWikiId=wikiuser.wikiId)
        askQuery = self.getAskQuery(askExtra)
        if self.debug:
            print(askQuery)
        startTime = time.time()
        entityName = self.entityManager.entityName
        records = self.wikiPush.formatQueryResult(askQuery, self.wikiClient, entityName=entityName)
        elapsed = time.time() - startTime
        if profile:
            print("query of %d %s records took %5.1f s" % (len(records), entityName, elapsed))
        return records

    def getLoDfromWikiFileManager(self, wikiFileManager:WikiFileManager):
        '''

        Args:
            wikiFileManager(WikiFileManager):
        Return:
            list of dicts
        '''
        if not self.wikiFileManager and wikiFileManager:
            self.wikiFileManager = wikiFileManager
        wikiFileDict = self.wikiFileManager.getAllWikiFiles()
        lod=self.getLoDfromWikiFiles(wikiFileDict.values())
        for record in lod:
            pageTitle=record.get("pageTitle")
            wikiFile=wikiFileDict.get(pageTitle)
            if isinstance(wikiFile, WikiFile):
                record["wikiMarkup"]=wikiFile.wikiText
        return lod

    def getLoDfromWikiFiles(self, wikiFileList: list):
        '''
        Convert given wikiFiles to LoD

        Args:
            wikiFileList(list):

        Return:
            list of dicts
        '''
        templateName = self.entityManager.clazz.templateName
        wikiSonLod = WikiFileManager.convertWikiFilesToLOD(wikiFileList, templateName)
        lod = self.normalizeLodFromWikiSonToLod(wikiSonLod)
        return lod

    def fromWikiFileManager(self, wikiFileManager):
        """
        initialize me from the given WikiFileManager
        """
        lod=self.getLoDfromWikiFileManager(wikiFileManager)
        self.entityManager.fromLoD(lod)

    def fromWikiFiles(self, wikiFileList: list):
        '''
        initialize me from the given list of wiki files
        '''
        lod=self.getLoDfromWikiFiles(wikiFileList)
        self.entityManager.fromLoD(lod)

    def fromSampleWikiSonLod(self, entityClass):
        '''
        get a list of dicts derived form the wikiSonSamples

        Returns:
            list: a list of dicts for my sampleWikiText in WikiSon notation
        '''
        wikiFileList = []
        for sampleWikiText in entityClass.getSampleWikiTextList():
            pageTitle = None
            wikiFile = WikiFile(name=pageTitle, wikiText=sampleWikiText)
            wikiFileList.append(wikiFile)
        self.fromWikiFiles(wikiFileList)

    def normalizeLodFromWikiSonToLod(self, wikiSonRecords: list) -> list:
        '''
        normalize the given LOD to the properties in the propertyLookupList

        Args:
            wikiSonRecords(list): the list of dicts to normalize/convert

        Return:
            list: a list of dict to retrieve entities from
        '''
        lod = []
        lookup=None
        if callable(getattr(self.entityManager.clazz, "getPropertyLookup", None)):
            lookup=self.entityManager.clazz.getTemplateParamLookup()
        if lookup:
            for record in wikiSonRecords:
                if not isinstance(record, dict):
                    continue
                normalizedDict = SMWEntity.updateDictKeys(record, lookup)
                # make sure the pageTitle survives (it is not in the property mapping ...)
                if "pageTitle" in record:
                    normalizedDict["pageTitle"] = record["pageTitle"]
                lod.append(normalizedDict)
        return lod

    def updateEntitytoWiki(self, entity, overwrite:bool = True, targetWiki=None, uploadToWikiCallback:Callable = None):
        if hasattr(entity, 'pageTitle'):
            self.interlinkEnititesWithWikiMarkupFile()
            entity.smwHandler.saveToWikiText(overwrite=overwrite)
            wikiFile = self.wikiFileManager.getWikiFile(entity.pageTitle)
            if uploadToWikiCallback is not None:
                uploadToWikiCallback(wikiFile,targetWiki)

    def interlinkEnititesWithWikiMarkupFile(self, useCacheIfPresent:bool=False):
        '''
        Assigns the correspondingWikiFile to the SMWEntity
        '''
        for entity in self.entityManager.getList():
            if hasattr(entity, "pageTitle"):
                pageTitle=entity.pageTitle
                if hasattr(entity, "smwHandler") and isinstance(getattr(entity, "smwHandler"), SMWEntity):
                    smwHandler=entity.smwHandler
                    if useCacheIfPresent and hasattr(entity, "wikiMarkup"):
                        smwHandler.wikiFileManager=self.wikiFileManager
                        smwHandler._wikiFile=None
                    else:
                        wikiFile=self.wikiFileManager.getWikiFile(entity.pageTitle)
                        smwHandler.wikiFile=wikiFile


'''
Created on 2021-07-28

@author: th
'''
from corpus.event import EventSeriesManager, EventSeries, Event, EventManager
from lodstorage.storageconfig import StorageConfig
from corpus.datasources.dblpxml import DblpXml
from corpus.eventcorpus import EventDataSource, EventDataSourceConfig
import re
from datetime import datetime

class Dblp(EventDataSource):
    '''
    scientific events from https://dblp.org
    '''
    sourceConfig = EventDataSourceConfig(lookupId="dblp", name="dblp", url='https://dblp.org/', title='dblp computer science bibliography', tableSuffix="dblp")
    
    
    def __init__(self):
        '''
        constructor
        '''
        super().__init__(DblpEventManager(), DblpEventSeriesManager(), Dblp.sourceConfig)
       
    @classmethod
    def setPatterns(cls):
        if hasattr(cls,"dayPattern"):
            return
        cls.dayPattern=r""
        cls.dayPatternWithSuffix=r""
        delim=""
        for day in range(1,31):
            # https://stackoverflow.com/a/739266/1497139
            cls.dayPattern=cls.dayPattern+f"{delim}{day}"
            delim="|"
        cls.monthPattern=r"January|February|March|April|May|June|July|August|September|October|November|December"
        cls.yearPattern=r"([12][0-9]{3})"
        cls.ws=r"\s+"
        cls.dateRangePattern=f"^\s*({cls.dayPattern})[-]({cls.dayPattern})(st|nd|rd|th)?{cls.ws}({cls.monthPattern}){cls.ws}({cls.yearPattern})\s*[.]?$"        


    @classmethod
    def strToDate(cls,dateStr):
        '''
        Args:
            dateStr(str): the string to convert
            
        Return:
            datetime: the date
        '''
        d=None
        try:
            d = datetime.strptime(dateStr, '%d %B %Y')
        except ValueError as _ve:
            
            pass
        return d
    
    @classmethod
    def getDateRangeFromTitle(cls,title:str):
        dateRange={}
        if title is not None:
            parts=title.split(",")
            datePartStr=parts[len(parts)-1]
            dateRange=Dblp.getDateRange(datePartStr)
        return dateRange 
    
    @classmethod
    def getDateRange(cls,dateStr):
        '''
        given a dblp date string create a date range
        
        Args:
            dateStr(str): the date string to analyze
            
        Returns:
            dict: containing year, startDate, endDate
        examples:
            18-21 September 2005

        '''
        cls.setPatterns()
        result={}
        if dateStr is not None:
            yearOnly=re.search(f"^{cls.yearPattern}$",dateStr)
            dateRangeMatch=re.search(cls.dateRangePattern,dateStr)
            if yearOnly: 
                result['year']=int(yearOnly.group(1))
            elif dateRangeMatch:      
                fromDay=dateRangeMatch.group(1)
                toDay=dateRangeMatch.group(2)
                month=dateRangeMatch.group(4)
                year=dateRangeMatch.group(5)
                startDateStr=f"{fromDay} {month} {year}"
                toDateStr=f"{toDay} {month} {year}"   
                startDate=cls.strToDate(startDateStr)   
                endDate=cls.strToDate(toDateStr)
                if startDate and endDate: 
                    result['startDate']=startDate
                    result['endDate']=endDate
        if 'startDate' in result:
                result['year']=result['startDate'].year
        return result

        
class DblpEvent(Event):
    '''
    a Dblp Event
    
    Example event: https://dblp.org/db/conf/aaai/aaai2020.html
    '''

    def __init__(self):
        '''constructor '''
        super().__init__()
        pass
        
    
    @staticmethod
    def postProcessLodRecord(rawEvent:dict):
        '''
        fix the given raw Event
        
        Args:
            rawEvent(dict): the raw event record to fix
        '''
        if 'url' in rawEvent:
            rawEvent["url"] = f"https://dblp.org/{rawEvent['url']}" 
        if 'title' in rawEvent:   
            title=rawEvent["title"]
            dateRange=Dblp.getDateRangeFromTitle(title)
            for key, value in dateRange.items():
                rawEvent[key]=value
        if "year" in rawEvent:
            # set year to integer value
            yearStr = rawEvent['year']
            year = None
            try:
                year = int(yearStr)
            except Exception as _ne:
                pass
            rawEvent['year'] = year
            # if there is a booktitle create acronym
            if "booktitle" in rawEvent:
                booktitle = rawEvent['booktitle']
                if booktitle is not None and year is not None:
                    acronym = f"{booktitle} {year}"
                    rawEvent["acronym"] = acronym 
        doiprefix = "https://doi.org/"
        if 'ee' in rawEvent:
            ees = rawEvent['ee']
            if ees:
                for ee in ees.split(","):
                    if ee.startswith(doiprefix):
                        doi = ee.replace(doiprefix, "")
                        rawEvent["doi"] = doi 


class DblpEventSeries(EventSeries):
    '''
    a Dblp Event Series
    
    Example event series: https://dblp.org/db/conf/aaai/index.html
    '''

    def __init__(self):
        '''constructor '''
        super().__init__()
        pass
        

class DblpEventManager(EventManager):
    '''
    dblp event access (in fact proceedings)
    
    Example event: https://dblp.org/db/conf/aaai/aaai2020.html
    
    '''
    cacheOnly=False

    def __init__(self, config: StorageConfig=None):
        '''
        Constructor
        '''
        super(DblpEventManager, self).__init__(name="DblpEvents", sourceConfig=Dblp.sourceConfig, clazz=DblpEvent, config=config)

    pass

    def configure(self):
        '''
        configure me
        '''
        withProgress = False
        if DblpEventManager.cacheOnly:
            return
        if not hasattr(self, "dblpXml"): 
            self.dblpXml = DblpXml()
            self.dblpXml.warnFullSize()
            withProgress = True
        self.sqlDb = self.dblpXml.getXmlSqlDB(showProgress=withProgress)
        if not hasattr(self, "getListOfDicts"):
            self.getListOfDicts = self.getLoDfromDblp

    def getLoDfromDblp(self) -> list:
        '''
        get the LoD for the event series
            
        Return:
            list: the list of dict with my series data

        '''
        query = """select conf as series,title,year,url,booktitle,series as publicationSeries,ee,isbn,mdate,key as eventId
        from proceedings 
        order by series,year"""
        listOfDicts = self.sqlDb.query(query)
        self.setAllAttr(listOfDicts, "source", "dblp")
        self.postProcessLodRecords(listOfDicts)
        return listOfDicts


class DblpEventSeriesManager(EventSeriesManager):
    '''
    dblp event series access
    Example event series: https://dblp.org/db/conf/aaai/index.html

    dblp provides regular dblp xml dumps
    '''

    def __init__(self, config: StorageConfig=None):
        '''
        Constructor
        '''
        super().__init__(name="DblpEventSeries", sourceConfig=Dblp.sourceConfig, clazz=DblpEventSeries, config=config)
        
    def configure(self):
        '''
        configure me
        '''
        withProgress = False
        if DblpEventManager.cacheOnly:
            return
        if not hasattr(self, "dblpXml"): 
            self.dblpXml = DblpXml()
            self.dblpXml.warnFullSize()
            withProgress = True
        self.sqlDb = self.dblpXml.getXmlSqlDB(showProgress=withProgress)
        if not hasattr(self, "getListOfDicts"):
            self.getListOfDicts = self.getLoDfromDblp

    def getLoDfromDblp(self) -> list:
        '''

        get the list of dicts for the event data
            
        Return:
            list: the list of dict with my event data

        '''
        query = """select conf as acronym,conf as eventSeriesId,count(*) as count,min(year) as minYear,max(year) as maxYear
        from proceedings 
        where acronym is not null
        group by acronym
        order by 2 desc"""
        listOfDicts = self.sqlDb.query(query)
        self.setAllAttr(listOfDicts, "source", "dblp")
        self.postProcessLodRecords(listOfDicts)
        return listOfDicts
